CREATE user sampleschema identified by rfidic account lock;
GRANT SELECT ON viewschema.epclist   TO sampleschema ;
GRANT SELECT ON viewschema.event     TO sampleschema ;
GRANT SELECT ON viewschema.translist TO sampleschema ;
GRANT SELECT ON viewschema.location  TO sampleschema ;

--
-- EPC Lifecycle report.
--
CREATE VIEW sampleschema.epc_lifecycle (epc,source,type,document,eventtime,
                                        bizstep,readpoint,disposition,parent_id,
                                        action,biztransactiontype,bizlocationname,
                                        readpointname) AS
  SELECT epclist.epc                     epc,
         event.source                    source,
         event.type                      type,
         event.document                  document,
         event.eventtime                 eventtime,
         event.bizstep                   bizstep,
         event.readpoint                 readpoint,
         event.disposition               disposition,         
         event.parentid                  parent_id,
         event.action                    action,
         (SELECT biztransactiontype    FROM  viewschema.translist
                 WHERE event.event_id = event_id) biztransactiontype,
         (SELECT sslta                 FROM  viewschema.location
                 WHERE event.bizlocation   = external_id AND
                       vocabulary = 'urn:epcglobal:epcis:vtype:BusinessLocation') bizlocationname,
         (SELECT name                  FROM  viewschema.location
                 WHERE event.readpoint = external_id AND
                       vocabulary   = 'urn:epcglobal:epcis:vtype:ReadPoint') readpointname
  FROM viewschema.epclist                epclist, 
       viewschema.event                  event
  WHERE
       event.event_id       = epclist.event_id  AND
       event.parentid IS NULL                   AND
       event.source   IS NULL;

--------------------------------------------------------------------------------
-- The following SQL creates the 'missing reads' view from the 
-- epc_lifecycle view.
--
-- EPC Lifecycle view:
--   EPC         TYPE       DATE
--   ----------- ---------- ----------
--           123 commission 04/05/2007
--           123 shipped    04/06/2007
--           123 received   04/07/2007
--           456 commission 03/26/2007
--           456 received   03/27/2007
--
-- Missing reads view:
--   EPC         COMMISSION SHIPPED    RECEIVED
--   ----------- ---------- ---------- ----------
--           123 04/05/2007 04/06/2007 04/07/2007
--           456 03/26/2007 -          03/27/2007
--------------------------------------------------------------------------------

CREATE VIEW sampleschema.distinct_epc (epc) AS
     (select distinct(epc) epc from sampleschema.epc_lifecycle);

-- Define a table for each transaction type.
CREATE VIEW sampleschema.event_commission (epc, eventtime) AS 
     (select epc, eventtime 
      from sampleschema.epc_lifecycle where bizstep = 'urn:epcglobal:hls:bizstep:commission');

CREATE VIEW sampleschema.event_shipped    (epc, eventtime) AS
     (select epc, eventtime 
      from sampleschema.epc_lifecycle where bizstep = 'urn:epcglobal:fmcg:bizstep:shipped');

CREATE VIEW sampleschema.event_receiving  (epc, eventtime) AS
     (select epc, eventtime 
      from sampleschema.epc_lifecycle where bizstep = 'urn:epcglobal:fmcg:bizstep:receiving');

-- Define a table for each transaction type with a row for every EPC even those EPCs which don't have this transaction type.
CREATE VIEW sampleschema.event_commission2 (epc, eventtime) as 
     (select          distinct_epc.epc, eventtime 
      from            sampleschema.distinct_epc     distinct_epc 
      left join sampleschema.event_commission event_commission 
      on distinct_epc.epc = event_commission.epc);

CREATE VIEW sampleschema.event_shipped2    (epc, eventtime) as 
     (select          distinct_epc.epc, eventtime 
      from            sampleschema.distinct_epc  distinct_epc 
      left join sampleschema.event_shipped event_shipped 
      on distinct_epc.epc = event_shipped.epc);

CREATE VIEW sampleschema.event_receiving2  (epc, eventtime) as 
     (select          distinct_epc.epc, eventtime 
      from            sampleschema.distinct_epc    distinct_epc
      left join sampleschema.event_receiving event_receiving 
      on distinct_epc.epc = event_receiving.epc);


-- Join the tables together such that there is 1 row per EPC with the commission/shipped/receiving information in columns.
CREATE VIEW sampleschema.missing_reads AS
  SELECT
    distinct_epc.epc             epc,
    event_commission2.eventtime  commission_time,
    event_shipped2.eventtime     shipped_time,
    event_receiving2.eventtime   receiving_time
  FROM
    sampleschema.distinct_epc      distinct_epc,
    sampleschema.event_commission2 event_commission2,
    sampleschema.event_shipped2    event_shipped2,
    sampleschema.event_receiving2  event_receiving2
  WHERE
    distinct_epc.epc = event_commission2.epc and
    distinct_epc.epc = event_shipped2.epc    and
    distinct_epc.epc = event_receiving2.epc;

COMMIT;

-- /////////////////////  Create the message publishing schema ////////////////////

create user publish identified by rfidic account lock;
GRANT DBA TO publish;

create table publish.pub_item_title
(
   title_id integer not null primary key,
   title_name varchar(255),
   title_hit integer not null
);

create table publish.pub_message
(msg_id integer NOT NULL primary key,
 corr_id varchar(200) not null,
 pub_code integer not null,
 pub_domain integer references publish.pub_item_title(title_id) on delete cascade
);

create table publish.pub_object
(
 obj_id integer not null primary key,
 obj_title integer references publish.pub_item_title(title_id) on delete cascade,
 msg_id integer not null references publish.pub_message(msg_id) on delete cascade
);

create table publish.pub_object_value
(
 obj_id integer not null references publish.pub_object(obj_id) on delete cascade,
 prop_title integer not null references publish.pub_item_title(title_id) on delete cascade,
 prop_value varchar(2048),
 prop_type integer
);
 
create view publish.timestamp_prop(msg_id, obj_type, obj_id, prop_name, timestamp_value) as
select O.msg_id, OT.title_name obj_type, O.obj_id, T.title_name prop_name, TO_TIMESTAMP(replace(replace(V.prop_value, 'Z'), 'T', ' '), 'YYYY-MM-DD HH24:MI:SS.FF3') timestamp_value
from publish.pub_item_title T,
     publish.pub_item_title OT,
     publish.pub_object_value V,
     publish.pub_object O
where T.title_id = V.prop_title and
      OT.title_id = O.obj_title and
      V.obj_id = O.obj_id and
      prop_type=3;

create view publish.int_prop(msg_id, obj_type, obj_id, prop_name, int_value) as
select O.msg_id, OT.title_name obj_type, O.obj_id, T.title_name prop_name, cast(V.prop_value as INTEGER) int_value
from publish.pub_item_title T,
     publish.pub_item_title OT,
     publish.pub_object_value V,
     publish.pub_object O
where T.title_id = V.prop_title and
      OT.title_id = O.obj_title and
      V.obj_id = O.obj_id and
      prop_type=2;

create view publish.string_prop(msg_id, obj_type, obj_id, prop_name, string_value) as
select O.msg_id, OT.title_name obj_type, O.obj_id, T.title_name prop_name, V.prop_value string_value
from publish.pub_item_title T,
     publish.pub_item_title OT,
     publish.pub_object_value V,
     publish.pub_object O
where T.title_id = V.prop_title and
      OT.title_id = O.obj_title and
      V.obj_id = O.obj_id and
      prop_type=1;

create view publish.notification_view(msg_id, timestamp, subscriber, alert_name, notif_config_name, subscription_id) as
select A.msg_id msg_id,
       A.timestamp_value timestamp, 
       B.string_value subscriber, 
       C.string_value alert_name,
       D.string_value notif_config_name, 
       E.int_value subscription_id      
from publish.timestamp_prop A, 
     publish.string_prop B, 
     publish.string_prop C,
     publish.string_prop D, 
     publish.int_prop E,
     publish.pub_message M
where A.obj_type = 'NotificationRecord' and A.prop_name='Timestamp' and
      B.obj_type = 'NotificationRecord' and B.prop_name='Subscriber' and
      C.obj_type = 'NotificationRecord' and C.prop_name='AlertName' and
      D.obj_type = 'NotificationRecord' and D.prop_name='NotificationConfigurationName' and
      E.obj_type = 'NotificationRecord' and E.prop_name='SubscriptionId' and
      A.msg_id = B.msg_id and 
      A.msg_id = C.msg_id and
      A.msg_id = D.msg_id and
      A.msg_id = E.msg_id and
      M.msg_id = A.msg_id and
      M.pub_domain in (
             select title_id
             from publish.pub_item_title
             where title_name='http://www.ibm.com/rfidic/alerts/notifications/1.1/audit/domain/simple');

CREATE SEQUENCE publish.pub_item_title___0;
CREATE SEQUENCE publish.pub_message___0;
CREATE SEQUENCE publish.pub_object___0;

commit;