/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.handlers.basic;

import org.w3c.dom.Node;

import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.event.failed.FailedEvent;
import com.ibm.rfidic.handlers.IHandler;
import com.ibm.rfidic.handlers.InterningException;
import com.ibm.rfidic.mdm.IElement;
import com.ibm.rfidic.mdm.MasterDataManager;
import com.ibm.rfidic.metadata.IAttributeMetaData;
import com.ibm.rfidic.metadata.IEntityMetaData;
import com.ibm.rfidic.metadata.MetaDataManager;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.ValueFactory;

/**
 * 
 * Handles interning product (epc)
 * 
 * This class does split epc codes into company.product.serial.  It interns
 * the whole epc code (urn:epc:tag:sgtin:4057000.123430.0000) by stripping off the serial 
 * number and looking up the using the rest for MDM lookup.
 * So it assumes that MasterData has the matching product data (without the serial number).
 * 
 * As an example external IDs in the masterdata need to be urn:epc:tag:sgtin:company.product
 * e.g. urn:epc:tag:sgtin:4057000.123430 will be the externai ID used for 
 * EPC urn:epc:tag:sgtin:4057000.123430.0000. 
 *
 */
public class SimpleGTINHandler implements IHandler {
	public static final String copyright = "(c) Copyright IBM Corporation 2006.";
	

    private MasterDataManager m_mdm;
    IEntityMetaData m_meta;
    
    public SimpleGTINHandler () throws Exception
    {
        m_mdm = MasterDataManager.getInstance();
		if (m_mdm == null)
			throw new RFIDICException("Unable to initialize MDM API.");
		m_meta = MetaDataManager.getInstance().getEPCISDocumentMetaData().getEventMetaData();
    }
        
    
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#intern(com.ibm.rfidic.metadata.IAttributeMetaData, org.w3c.dom.Node)
     */
    public IAttributeValue intern(IAttributeMetaData attributeMD, Node node)
            throws InterningException
    {     
        throw new InterningException("Handler does not handle XML fragments.");
    }
    
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#intern(com.ibm.rfidic.metadata.IAttributeMetaData, com.ibm.rfidic.value.IAttributeValue)
     */
    public IAttributeValue intern(IAttributeMetaData attributeMD, IAttributeValue value) throws InterningException
    {
        String epc = value.toString();
        if( epc == null ) return null;
        
        // strip out the serial number from the epc
        int lastDotPos = epc.lastIndexOf('.');
        
        // derive the external ID from the product epc
        String externalID=epc;
        if( lastDotPos >= 0 )
        {
            externalID= externalID.substring(0, lastDotPos);
        }
        
        IElement product = m_mdm.getProduct(externalID);
        if( product == null )
        {
        	// determine the type or vocabulary from the metadata or use default
            String type = ((attributeMD.getVocabularies() != null) && (attributeMD
                    .getVocabularies().length > 0)) ? attributeMD
                    .getVocabularies()[0] : "Product type";
            
            String error="The handler did not find a product with an SGTIN value of '"+epc+"' for the '"+attributeMD.getName()+"' attribute. "+
            "The external ID is '"+externalID+"'. Import the product.";
            
            throw new InterningException(error                    
                    , attributeMD.getName(), value, type,
                    FailedEvent.RCMissingProduct);
        }
        
        // check if the location is actually in the vocabulary that was specified in the 
        // metadata. Otherwise the event views won't work.
        String locationVocabulary=product.getVocabulary().getName();
        int matchedVocab=-1;
        String vocabs[]=attributeMD.getVocabularies();
        for (int i=0;i<vocabs.length;i++) {
            if (locationVocabulary.equals(vocabs[i]))
                matchedVocab=i;
        }        
        if (matchedVocab==-1) {
            String error="Product with external ID '"+externalID+"' was found in vocabulary '"+locationVocabulary+
            "', but the metadata does not include this for attribute '"+attributeMD.getName()+
            "'. Update the metadata.";
            throw new InterningException(error, attributeMD.getName(), value, vocabs[0],
                    FailedEvent.RCMissingProduct);
        }

        
        int intern = product.getId();
        return ValueFactory.getValue(intern);
    }
	
    /* (non-Javadoc)
     * @see com.ibm.rfidic.handlers.IHandler#unIntern(int)
     */
    public IAttributeValue unIntern(int internID) throws InterningException
    {
        // This handler doesn't preserve the serial number for the EPC, so it cannot be used for uninterning.
        // make sure you specify a column for the orginal value by specifying the 
        // UninternedColumnName property for the attributes where you use this handler. 
      throw new InterningException("SimpleGTINHandler cannot be used for uninterning. Ensure you have the UninternedColumnName property"+
              " set in the metadata.");
    }
        
}
