/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.coral8;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.rfidic.alerts.AlertsAdapter;
import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.alerts.coral8.stream.Field;
import com.ibm.rfidic.alerts.coral8.stream.Stream;
import com.ibm.rfidic.alerts.registry.RegistryException;
import com.ibm.rfidic.alerts.registry.RegistryManager;
import com.ibm.rfidic.common.epcis.EventConstants;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.rfidic.value.ValueFactory;

/**
 * @author vcrisan
 */
public class CaseEnteredFrontDoorAlert extends Entity implements ICoral8Alert
{
  public final static String READPOINT_PARAMETER_NAME = EventConstants.readPoint;
  public final static String EPC_PREFIX_PARAMETER_NAME = "epcPrefix";

  private IParameter[] inputParameters;

  private IParameter[] outputParameters;

  private ICoral8Engine coral8Engine;

  private Stream notificationStream;

  private ICoral8SubscriptionManager coral8SubscriptionManager;

  private RegistryManager registryManager;

  public CaseEnteredFrontDoorAlert()
  throws RegistryException
  {
    super("Case Entered Through Front Door",
    "Sample alert to notify when a case (or tote) enters the pharmacy through the front door (instead of the back door)");

    /*
     * Input parameters:
     * 
     * 1. Read point ID of front door
     * 
     * 2. EPC class of totes (of C2 drugs) entering the pharmacy
     */
    this.inputParameters = new IParameter[2];
    this.inputParameters[0] = new Parameter(READPOINT_PARAMETER_NAME, TypeFactory.getInstance().getStringType(255, 0), true, null);
    this.inputParameters[1] = new Parameter(EPC_PREFIX_PARAMETER_NAME, TypeFactory.getInstance().getStringType(255, 0), true, null);

    /*
     * Output parameters
     */
    this.outputParameters = new IParameter[5];
    this.outputParameters[0] = new Parameter(EventConstants.eventId, TypeFactory.getInstance().getIntegerType(), true, null);
    this.outputParameters[1] = new Parameter(EventConstants.eventType, TypeFactory.getInstance().getStringType(255, 0), true,
        null);
    this.outputParameters[2] = new Parameter(EventConstants.eventTime, TypeFactory.getInstance().getDateType(), true, null);
    this.outputParameters[3] = new Parameter(EventConstants.readPoint, TypeFactory.getInstance().getStringType(255, 0), true,
        null);
    this.outputParameters[4] = new Parameter(EventConstants.epc, TypeFactory.getInstance().getStringType(255, 0), true, null);

    coral8Engine = null;
    coral8SubscriptionManager = null;
    registryManager = RegistryManager.getInstance();
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#initialize()
   */
  public boolean initialize()
  {
    coral8Engine = AlertsAdapter.getInstance().getCoral8Engine();

    if (coral8Engine == null)
      return false;

    notificationStream = new Stream(coral8Engine, coral8Engine.generateUniqueOutStreamName("stream1"));

    notificationStream.addField(new Field(ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD, Field.INTEGER_TYPE));
    notificationStream.addField(new Field(EventConstants.eventId, Field.INTEGER_TYPE));
    notificationStream.addField(new Field(EventConstants.eventType, Field.STRING_TYPE));
    notificationStream.addField(new Field(EventConstants.eventTime, Field.TIME_TYPE));
    notificationStream.addField(new Field(EventConstants.readPoint, Field.STRING_TYPE));
    notificationStream.addField(new Field(EventConstants.epc, Field.STRING_TYPE));

    coral8SubscriptionManager = coral8Engine.generateSubscriptionManager(this);

    return true;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#getInputParameters()
   */
  public IParameter[] getInputParameters()
  {
    return inputParameters;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.framework.IAlert#getOutputParameters()
   */
  public IParameter[] getOutputParameters()
  {
    return outputParameters;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#initializeSubscription(com.ibm.rfidic.alerts.ISubscription)
   */
  public void addSubscription(ISubscription subscription)
  {
    if (coral8SubscriptionManager != null)
    {
      coral8SubscriptionManager.add(subscription);
    }
    else
    {
      System.err.println("The Coral8 subscription manager is not set so the subscription cannot be added.");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#cleanSubscription(com.ibm.rfidic.alerts.ISubscription)
   */
  public void removeSubscription(ISubscription subscription)
  {
    if (coral8SubscriptionManager != null)
    {
      coral8SubscriptionManager.remove(subscription);
    }
    else
    {
      System.err.println("The Coral8 subscription manager is not set so the subscription cannot be removed.");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.coral8.ICoral8Alert#processOutputMessage(java.util.Map)
   */
  public List processOutputMessage(Map valueMap) throws Exception
  {
    final List fields = notificationStream.getFields();
    final IParameterValue[] values = new IParameterValue[outputParameters.length];

    int parameterCount = 0;

    final Iterator iterator = fields.iterator();

    // first value is the subscription ID
    final int subscriptionId = ((Integer) valueMap.get(iterator.next())).intValue();
    final ISubscription subscription = (ISubscription) registryManager.getSubscriptionRegistry()
    .get(subscriptionId);

    while (iterator.hasNext())
    {
      final IField field = (IField) iterator.next();

      final Object value = valueMap.get(field);

      values[parameterCount] = new ParameterValue(outputParameters[parameterCount], ValueFactory.getValue(outputParameters[parameterCount].getType(),
          value));
      parameterCount++;
    }

    final List notifications = new ArrayList();
    final INotification notification = new Notification(subscription, values);

    notifications.add(notification);

    return notifications;
  }

  /**
   * Get the sequence of CCL statements for the Coral8 engine.
   * @param engine
   * @return the CCL statements separated by semicolons
   */
  public String getQueries(ICoral8Engine engine)
  {
    if (coral8SubscriptionManager == null)
    {
      return null;
    }

    final IStream eventStream = engine.getInEventStream();
    final IStream epcStream = engine.getInStreamForAttribute(EventConstants.epc);

    final StringBuffer query = new StringBuffer();

    query.append("CREATE WINDOW ");
    query.append("w_event ");
    query.append("SCHEMA (");
    query.append(EventConstants.eventId + " Integer, ");
    query.append(EventConstants.eventType + " String, ");
    query.append(EventConstants.eventTime + " Timestamp, ");
    query.append(EventConstants.readPoint + " String");
    query.append(") KEEP LAST;\n");

    query.append("INSERT INTO ");
    query.append("w_event ");
    query.append("(");
    query.append(EventConstants.eventId + ", ");
    query.append(EventConstants.eventType + ", ");
    query.append(EventConstants.eventTime + ", ");
    query.append(EventConstants.readPoint);
    query.append(") ");        
    query.append(" SELECT ");
    query.append("s." + EventConstants.eventId + ", ");
    query.append("s." + EventConstants.eventType + ", ");
    query.append("s." + EventConstants.eventTime + ", ");
    query.append("s." + EventConstants.readPoint);
    query.append(" FROM ");
    query.append(eventStream.getName() + " AS s;\n");

    query.append("INSERT INTO ");
    query.append(notificationStream.getName());
    query.append(" (");
    query.append(ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD + ", ");
    query.append(EventConstants.eventId +", ");
    query.append(EventConstants.eventType + ", ");
    query.append(EventConstants.eventTime + ", ");
    query.append(EventConstants.readPoint + ", ");
    query.append(EventConstants.epc);
    query.append(") ");        
    query.append("SELECT ");
    query.append("sw." + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD + ", ");
    query.append("w." + EventConstants.eventId + ", ");
    query.append("w." + EventConstants.eventType + ", ");
    query.append("w." + EventConstants.eventTime + ", ");
    query.append("w." + EventConstants.readPoint + ", ");
    query.append("s." + EventConstants.epc + " ");
    query.append("FROM ");
    query.append("w_event AS w, ");
    query.append(epcStream.getName() + " AS s, ");
    query.append(coral8SubscriptionManager.getWindowName() + " AS sw ");
    query.append("WHERE ");
    query.append("s." + EventConstants.eventId);
    query.append(" = ");
    query.append("w." + EventConstants.eventId);
    query.append(" and ");
    query.append("w." + EventConstants.readPoint);
    query.append(" = ");
    query.append("sw." + EventConstants.readPoint);
    query.append(" and ");
    query.append("s." + EventConstants.epc);
    query.append(" LIKE ");
    query.append("sw." + EPC_PREFIX_PARAMETER_NAME + "|| '%'");
    query.append(";\n");

    return query.toString();
  }

  /**
   * Get the output stream.
   * @param engine
   * @return the notification stream
   */
  public IStream getNotificationStream(ICoral8Engine engine)
  {
    return notificationStream;
  }

  /**
   * There are no local streams.
   * @param engine
   * @return null
   */
  public List getLocalStreams(ICoral8Engine engine)
  {
    return null;
  }

  /**
   * Nothing is done here since the processing is performed by the Coral8 engine.
   * @param event
   * @param subscriptions
   * @return null
   */
  public List evaluateConditions(IEvent event, List subscriptions)
  {
    return null;
  }

  /**
   * Register the alert with the Coral8 engine.
   */
  public void register()
  {
    AlertsAdapter.getInstance().getCoral8Engine().registerAlert(this);
  }

  /** 
   * Unregister the alert from the Coral8 engine.
   */
  public void unregister()
  {
    AlertsAdapter.getInstance().getCoral8Engine().unregisterAlert(this);
  }
}
