/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.coral8;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.rfidic.alerts.AlertsAdapter;
import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.alerts.coral8.stream.Field;
import com.ibm.rfidic.alerts.coral8.stream.Stream;
import com.ibm.rfidic.alerts.registry.RegistryException;
import com.ibm.rfidic.alerts.registry.RegistryManager;
import com.ibm.rfidic.common.epcis.EventConstants;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.rfidic.value.ValueFactory;

/**
 * @author vcrisan
 */
public class DrugNotInSafeAreaAlert extends Entity implements ICoral8Alert
{
  public final static String PARAMETER_NAME_BIZ_LOCATION_VAULT = "bizLocationVault";

  public final static String PARAMETER_NAME_EPC_PREFIX = "epcPrefix";

  public final static String PARAMETER_NAME_READ_POINT_RECEIVING_AREA = "readPointReceivingArea";

  public final static String PARAMETER_NAME_READ_POINT_DISPENSER = "readPointDispenser";

  public final static String PARAMETER_NAME_TIME_OUTSIDE_VAULT = "timeOutsideVault";

  public final static String LOCAL_STREAM_NAME_SAFE_AREA = "local_stream_drug_in_safe_area";

  public final static String LOCAL_STREAM_NAME_UNSAFE_AREA = "local_stream_drug_in_unsafe_area";

  public final static String WINDOW_NAME_KEEP_LAST_EPC = "window_keep_last_epc";

  public final static String WINDOW_NAME_KEEP_LAST_EVENT = "window_keep_last_event";

  private IParameter[] inputParameters;

  private IParameter[] outputParameters;

  private ICoral8Engine coral8Engine;

  private Stream notificationStream;

  private ICoral8SubscriptionManager coral8SubscriptionManager;

  private RegistryManager registryManager;

  public DrugNotInSafeAreaAlert()
  throws RegistryException
  {
    super("Drug Not In Safe Area",
    "Sample alert to notify when C2 drugs are in an unsafe area in the pharmacy for an exceedingly long time");

    /*
     * Input parameters:
     * 
     * 1. EPC class of the C2 drugs (if there is more than one EPC class, create separate alerts)
     * 
     * 2. Time interval that C2 drugs can be outside of the vault
     * 
     * 3. BizLocation ID of the vault
     * 
     * 4. Read point ID of the reader where the bottles are unpacked from the totes (the receiving place).
     * 
     * 5. Read point ID of the dispenser
     */
    this.inputParameters = new IParameter[5];
    this.inputParameters[0] = new Parameter(PARAMETER_NAME_EPC_PREFIX, TypeFactory.getInstance().getStringType(255, 0),
        true, null);
    this.inputParameters[1] = new Parameter(PARAMETER_NAME_TIME_OUTSIDE_VAULT, TypeFactory.getInstance()
        .getIntegerType(), true, null);
    this.inputParameters[2] = new Parameter(PARAMETER_NAME_BIZ_LOCATION_VAULT, TypeFactory.getInstance().getStringType(
        255, 0), true, null);
    this.inputParameters[3] = new Parameter(PARAMETER_NAME_READ_POINT_RECEIVING_AREA, TypeFactory.getInstance()
        .getStringType(255, 0), true, null);
    this.inputParameters[4] = new Parameter(PARAMETER_NAME_READ_POINT_DISPENSER, TypeFactory.getInstance()
        .getStringType(255, 0), true, null);

    /*
     * Output parameters:
     * 
     * 1. Event type
     * 
     * 2. Event time (need to figure out what time we exaclty want to show here)
     * 
     * 3. EPC of the drug that is in an unsafe place
     * 
     * 4. Read point ID of the reader where the drug with this EPC was last seen
     */
    this.outputParameters = new IParameter[5];
    this.outputParameters[0] = new Parameter(EventConstants.eventId, TypeFactory.getInstance().getIntegerType(), true,
        null);
    this.outputParameters[1] = new Parameter(EventConstants.eventType, TypeFactory.getInstance().getStringType(255, 0),
        true, null);
    this.outputParameters[2] = new Parameter(EventConstants.eventTime, TypeFactory.getInstance().getDateType(), true,
        null);
    this.outputParameters[3] = new Parameter(EventConstants.readPoint, TypeFactory.getInstance().getStringType(255, 0),
        true, null);
    this.outputParameters[4] = new Parameter(EventConstants.epc, TypeFactory.getInstance().getStringType(255, 0), true,
        null);

    coral8Engine = null;
    coral8SubscriptionManager = null;
    registryManager = RegistryManager.getInstance();
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#initialize()
   */
  public boolean initialize()
  {
    coral8Engine = AlertsAdapter.getInstance().getCoral8Engine();

    if (coral8Engine == null)
      return false;

    notificationStream = new Stream(coral8Engine, coral8Engine.generateUniqueOutStreamName("stream1"));

    notificationStream.addField(new Field(ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD, Field.INTEGER_TYPE));
    notificationStream.addField(new Field(EventConstants.eventId, Field.INTEGER_TYPE));
    notificationStream.addField(new Field(EventConstants.eventType, Field.STRING_TYPE));
    notificationStream.addField(new Field(EventConstants.eventTime, Field.TIME_TYPE));
    notificationStream.addField(new Field(EventConstants.readPoint, Field.STRING_TYPE));
    notificationStream.addField(new Field(EventConstants.epc, Field.STRING_TYPE));

    coral8SubscriptionManager = coral8Engine.generateSubscriptionManager(this);

    return true;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#getInputParameters()
   */
  public IParameter[] getInputParameters()
  {
    return inputParameters;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.framework.IAlert#getOutputParameters()
   */
  public IParameter[] getOutputParameters()
  {
    return outputParameters;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#initializeSubscription(com.ibm.rfidic.alerts.ISubscription)
   */
  public void addSubscription(ISubscription subscription)
  {
    if (coral8SubscriptionManager != null)
    {
      coral8SubscriptionManager.add(subscription);
    }
    else
    {
      System.err.println("The Coral8 subscription manager is not set so the subscription cannot be added.");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.IAlert#cleanSubscription(com.ibm.rfidic.alerts.ISubscription)
   */
  public void removeSubscription(ISubscription subscription)
  {
    if (coral8SubscriptionManager != null)
    {
      coral8SubscriptionManager.remove(subscription);
    }
    else
    {
      System.err.println("The Coral8 subscription manager is not set so the subscription cannot be removed.");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.coral8.ICoral8Alert#processOutputMessage(java.util.Map)
   */
  public List processOutputMessage(Map valueMap) throws Exception
  {
    final List fields = notificationStream.getFields();
    final IParameterValue[] values = new IParameterValue[outputParameters.length];

    int parameterCount = 0;

    final Iterator iterator = fields.iterator();

    // first value is the subscription ID
    final int subscriptionId = ((Integer) valueMap.get(iterator.next())).intValue();
    final ISubscription subscription = (ISubscription) registryManager.getSubscriptionRegistry().get(subscriptionId);

    while (iterator.hasNext())
    {
      final IField field = (IField) iterator.next();

      final Object value = valueMap.get(field);

      values[parameterCount] = new ParameterValue(outputParameters[parameterCount], ValueFactory
          .getValue(outputParameters[parameterCount].getType(), value));
      parameterCount++;
    }

    final List notifications = new ArrayList();
    final INotification notification = new Notification(subscription, values);

    notifications.add(notification);

    return notifications;
  }

  private IStream getStreamForFieldName(Collection streams, String fieldName)
  {
    if (streams != null)
    {
      for (Iterator streamIterator = streams.iterator(); streamIterator.hasNext();)
      {
        final IStream stream = (IStream) streamIterator.next();
        for (Iterator fieldIterator = stream.getFields().iterator(); fieldIterator.hasNext();)
        {
          final IField field = (IField) fieldIterator.next();
          if ((field != null) && (field.getName() != null) && field.getName().equals(fieldName))
          {
            return stream;
          }
        }
      }
    }

    /*
     * Not found.
     */
    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.coral8.ICoral8Alert#getQueries(com.ibm.rfidic.alerts.coral8.ICoral8Engine)
   */
  public String getQueries(ICoral8Engine engine)
  {
    if (coral8SubscriptionManager == null)
    {
      return null;
    }

    final String streamNameEvent = engine.getInEventStream().getName();
    final String streamNameEPC = getStreamForFieldName(engine.getInEventStreams(), EventConstants.epc).getName();
    final String streamNameChildEPC = getStreamForFieldName(engine.getInEventStreams(), EventConstants.childEPC)
        .getName();
    final String streamNameSafe = LOCAL_STREAM_NAME_SAFE_AREA;
    final String streamNameUnsafe = LOCAL_STREAM_NAME_UNSAFE_AREA;
    final String streamNameNotification = notificationStream.getName();
    final String windowNameSubscription = coral8SubscriptionManager.getWindowName();
    final String windowNameLastEPC = WINDOW_NAME_KEEP_LAST_EPC;
    final String windowNameLastEvent = WINDOW_NAME_KEEP_LAST_EVENT;

    final StringBuffer ccl = new StringBuffer();

    /*
     * Add to unsafe zone all Child EPCs of totes that occur in the Receiving area of the pharmacy. The bottles are
     * taken out of the tote, so the event is an AggregationEvent.
     * 
     * INPUT:
     * 
     * Aggregation event (bottles with C2 drugs are taken out of the tote/case)
     * 
     * Get all child EPCs of the tote
     * 
     * Read point is receiving area
     */

    ccl.append("CREATE WINDOW ");
    ccl.append(windowNameLastEPC);
    ccl.append(" SCHEMA (");
    ccl.append(EventConstants.eventId + " Integer");
    ccl.append(", ");
    ccl.append(EventConstants.epc + " String");
    ccl.append(") KEEP LAST");
    ccl.append(";\n");

    ccl.append("INSERT INTO ");
    ccl.append(windowNameLastEPC);
    ccl.append(" (");
    ccl.append(EventConstants.eventId);
    ccl.append(", ");
    ccl.append(EventConstants.epc);
    ccl.append(") ");
    ccl.append("SELECT ");
    ccl.append("s." + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("s." + EventConstants.epc);
    ccl.append(" FROM ");
    ccl.append(streamNameEPC + " AS s");
    ccl.append(";\n");

    ccl.append("CREATE WINDOW ");
    ccl.append(windowNameLastEvent);
    ccl.append(" SCHEMA (");
    ccl.append(EventConstants.eventId + " Integer");
    ccl.append(", ");
    ccl.append(EventConstants.eventType + " String");
    ccl.append(", ");
    ccl.append(EventConstants.eventTime + " Timestamp");
    ccl.append(", ");
    ccl.append(EventConstants.readPoint + " String");
    ccl.append(", ");
    ccl.append(PARAMETER_NAME_EPC_PREFIX + " String");
    ccl.append(") KEEP LAST");
    ccl.append(";\n");

    ccl.append("INSERT INTO ");
    ccl.append(windowNameLastEvent);
    ccl.append(" (");
    ccl.append(EventConstants.eventId);
    ccl.append(", ");
    ccl.append(EventConstants.eventType);
    ccl.append(", ");
    ccl.append(EventConstants.eventTime);
    ccl.append(", ");
    ccl.append(EventConstants.readPoint);
    ccl.append(") ");
    ccl.append("SELECT ");
    ccl.append("s." + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("s." + EventConstants.eventType);
    ccl.append(", ");
    ccl.append("s." + EventConstants.eventTime);
    ccl.append(", ");
    ccl.append("s." + EventConstants.readPoint);
    ccl.append(" FROM ");
    ccl.append(streamNameEvent + " AS s");
    ccl.append(";\n");

    /*
     * QUERY 1: In the Receiving Area, totes are unpacked to get access to the class-2 drug bottles. The EPC of a tuple
     * is a child EPC (bottle) of the parent's EPC (tote).
     * 
     * Destination: Local stream "UNSAFE"
     * 
     * Parameters:
     * 
     * 1. Readpoint: <Read point ID for receiving area>
     * 
     * 2. Event type: AggregationEvent
     * 
     * NOTE: According to the Coral8 compiler, an error occurs that the "epc" column is not in the stream when this
     * stream is used in a query, which should not be the case since we It is not quite clear why we created the stream
     * with a given schema that includes the EPC column. when we explicitly map the name of the childEPC column name to
     * the EPC column name and do the same for all other columns, there is no such error.
     * 
     * Example Error message:
     * 
     * In Module "/AlertsProject" line 15 col 168 in RFIDIC CPLC2034: Error: Column 'epc' is not in stream 's'. "INSERT
     * INTO out_stream1_1 (subscriptionId, event_id, eventType, eventTime, readPoint, epc) SELECT s.subscriptionId,
     * s.event_id, s.eventType, s.eventTime, s.readPoint, s.epc FROM local_stream_drug_in_unsafe_area AS s;" ^ Module
     * AlertsProject from C:/DOCUME~1/ADMINI~1/LOCALS~1/Temp/tmp20442.ccl. "/AlertsProject", line 9, column 13: Error:
     * Result schema does not match schema automatically defined here.
     * 
     */
    ccl.append("INSERT INTO ");
    ccl.append(streamNameUnsafe);
    ccl.append(" SELECT ");
    ccl.append("ws." + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD + " AS "
        + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventId + " AS " + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventType + " AS " + EventConstants.eventType);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventTime + " AS " + EventConstants.eventTime);
    ccl.append(", ");
    ccl.append("we." + EventConstants.readPoint + " AS " + EventConstants.readPoint);
    ccl.append(", ");
    ccl.append("s." + EventConstants.childEPC + " AS " + EventConstants.epc);
    ccl.append(" FROM ");
    ccl.append(streamNameChildEPC + " AS s");
    ccl.append(", ");
    ccl.append(windowNameLastEvent + " AS we");
    ccl.append(", ");
    ccl.append(windowNameSubscription + " AS ws");
    ccl.append(" WHERE ");
    ccl.append("we." + EventConstants.readPoint);
    ccl.append(" = ");
    ccl.append("ws." + PARAMETER_NAME_READ_POINT_RECEIVING_AREA);
    ccl.append(" AND ");
    ccl.append("we." + EventConstants.eventType);
    ccl.append(" = ");
    ccl.append("'" + EventConstants.AggregationEvent + "'");
    ccl.append(" AND ");
    ccl.append("s." + EventConstants.eventId);
    ccl.append(" = ");
    ccl.append("we." + EventConstants.eventId);
    ccl.append(" AND ");
    ccl.append("s." + EventConstants.childEPC);
    ccl.append(" LIKE ");
    ccl.append("ws." + PARAMETER_NAME_EPC_PREFIX + "|| '%'");
    ccl.append(";\n");

    /*
     * QUERY 2: Any event which is not from the bottle dispenser and not from the vault.
     * 
     * Destination: Local stream "UNSAFE"
     * 
     * Parameters:
     * 
     * 1. Readpoint: <Read point ID for dispenser>
     * 
     * 2. Business location: <Business location of vault>
     */
    ccl.append("INSERT INTO ");
    ccl.append(streamNameUnsafe);
    ccl.append(" SELECT ");
    ccl.append("ws." + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventType);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventTime);
    ccl.append(", ");
    ccl.append("we." + EventConstants.readPoint);
    ccl.append(", ");
    ccl.append("s." + EventConstants.epc);
    ccl.append(" FROM ");
    ccl.append(streamNameEPC + " AS s");
    ccl.append(", ");
    ccl.append(windowNameLastEvent + " AS we");
    ccl.append(", ");
    ccl.append(windowNameSubscription + " AS ws");
    ccl.append(" WHERE ");
    ccl.append("we." + EventConstants.readPoint);
    ccl.append(" <> ");
    ccl.append("ws." + PARAMETER_NAME_READ_POINT_DISPENSER);
    ccl.append(" AND ");
    ccl.append("we." + EventConstants.readPoint);
    ccl.append(" <> ");
    ccl.append("ws." + PARAMETER_NAME_BIZ_LOCATION_VAULT);
    ccl.append(";\n");

    /*
     * QUERY 3: Any event which is from the vault.
     * 
     * Destination: Local stream "SAFE"
     * 
     * Parameters:
     * 
     * 1. Business location: <Business location of vault>
     */
    ccl.append("INSERT INTO ");
    ccl.append(streamNameSafe);
    ccl.append(" SELECT ");
    ccl.append("ws." + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD);
    ccl.append(", ");
    ccl.append("s." + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("s." + EventConstants.eventType);
    ccl.append(", ");
    ccl.append("s." + EventConstants.eventTime);
    ccl.append(", ");
    ccl.append("s." + EventConstants.readPoint);
    ccl.append(", ");
    ccl.append("we." + EventConstants.epc);
    ccl.append(" FROM ");
    ccl.append(streamNameEvent + " AS s");
    ccl.append(", ");
    ccl.append(windowNameLastEPC + " AS we");
    ccl.append(", ");
    ccl.append(windowNameSubscription + " AS ws");
    ccl.append(" WHERE ");
    ccl.append("we." + EventConstants.eventId);
    ccl.append(" = ");
    ccl.append("s." + EventConstants.eventId);
    ccl.append(" AND ");
    ccl.append("s." + EventConstants.bizLocation);
    ccl.append(" = ");
    ccl.append("ws." + PARAMETER_NAME_BIZ_LOCATION_VAULT);
    ccl.append(";\n");

    /*
     * QUERY 4: At the bottle dispenser, bottles are returned for recycling.
     * 
     * Destination: Local stream "SAFE"
     * 
     * Parameters:
     * 
     * 1. Readpoint: <Read point ID for dispenser>
     */
    ccl.append("INSERT INTO ");
    ccl.append(streamNameSafe);
    ccl.append(" SELECT ");
    ccl.append("ws." + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventType);
    ccl.append(", ");
    ccl.append("we." + EventConstants.eventTime);
    ccl.append(", ");
    ccl.append("we." + EventConstants.readPoint);
    ccl.append(", ");
    ccl.append("s." + EventConstants.epc);
    ccl.append(" FROM ");
    ccl.append(streamNameEPC + " AS s");
    ccl.append(", ");
    ccl.append(windowNameLastEvent + " AS we");
    ccl.append(", ");
    ccl.append(windowNameSubscription + " AS ws");
    ccl.append(" WHERE ");
    ccl.append("we." + EventConstants.readPoint);
    ccl.append(" = ");
    ccl.append("ws." + PARAMETER_NAME_READ_POINT_DISPENSER);
    ccl.append(";\n");

    /*
     * OUTPUT STREAM with pattern query
     */
    ccl.append("INSERT INTO ");
    ccl.append(streamNameNotification);
    ccl.append(" (");
    ccl.append(ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD);
    ccl.append(", ");
    ccl.append(EventConstants.eventId);
    ccl.append(", ");
    ccl.append(EventConstants.eventType);
    ccl.append(", ");
    ccl.append(EventConstants.eventTime);
    ccl.append(", ");
    ccl.append(EventConstants.readPoint);
    ccl.append(", ");
    ccl.append(EventConstants.epc);
    ccl.append(") ");
    ccl.append("SELECT ");
    ccl.append("u." + ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD);
    ccl.append(", ");
    ccl.append("u." + EventConstants.eventId);
    ccl.append(", ");
    ccl.append("u." + EventConstants.eventType);
    ccl.append(", ");
    ccl.append("u." + EventConstants.eventTime);
    ccl.append(", ");
    ccl.append("u." + EventConstants.readPoint);
    ccl.append(", ");
    ccl.append("u." + EventConstants.epc);
    ccl.append(" FROM ");
    ccl.append(streamNameSafe + " AS s");
    ccl.append(", ");
    ccl.append(streamNameUnsafe + " AS u");
    ccl.append(" MATCHING [5 SECONDS : u, !s]");
    ccl.append(" ON ");
    ccl.append("u." + EventConstants.epc);
    ccl.append(" = ");
    ccl.append("s." + EventConstants.epc);
    ccl.append(";\n");

    return ccl.toString();
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.coral8.ICoral8Alert#getNotificationStream(com.ibm.rfidic.alerts.coral8.Coral8Engine)
   */
  public IStream getNotificationStream(ICoral8Engine engine)
  {
    return notificationStream;
  }

  /*
   * (non-Javadoc)
   * 
   * @see com.ibm.rfidic.alerts.coral8.ICoral8Alert#getLocalStreams(com.ibm.rfidic.alerts.coral8.Coral8Engine)
   */
  public List getLocalStreams(ICoral8Engine engine)
  {
    final List list = new ArrayList();

    if (coral8SubscriptionManager == null)
    {
      return null;
    }

    coral8Engine = AlertsAdapter.getInstance().getCoral8Engine();

    if (coral8Engine == null)
    {
      return null;
    }

    Stream stream = null;

    stream = new Stream(coral8Engine, LOCAL_STREAM_NAME_SAFE_AREA);
    stream.addField(new Field(ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD, Field.INTEGER_TYPE));
    stream.addField(new Field(EventConstants.eventId, Field.INTEGER_TYPE));
    stream.addField(new Field(EventConstants.eventType, Field.STRING_TYPE));
    stream.addField(new Field(EventConstants.eventTime, Field.TIME_TYPE));
    stream.addField(new Field(EventConstants.readPoint, Field.STRING_TYPE));
    stream.addField(new Field(EventConstants.epc, Field.STRING_TYPE));
    list.add(stream);

    stream = new Stream(coral8Engine, LOCAL_STREAM_NAME_UNSAFE_AREA);
    stream.addField(new Field(ICoral8SubscriptionManager.SUBSCRIPTION_ID_FIELD, Field.INTEGER_TYPE));
    stream.addField(new Field(EventConstants.eventId, Field.INTEGER_TYPE));
    stream.addField(new Field(EventConstants.eventType, Field.STRING_TYPE));
    stream.addField(new Field(EventConstants.eventTime, Field.TIME_TYPE));
    stream.addField(new Field(EventConstants.readPoint, Field.STRING_TYPE));
    stream.addField(new Field(EventConstants.epc, Field.STRING_TYPE));
    list.add(stream);

    return list;
  }

  public List evaluateConditions(IEvent event, List subscriptions)
  {
    return null;
  }

  /** 
   * Register the alert with the Coral8 engine.
   */
  public void register()
  {
    AlertsAdapter.getInstance().getCoral8Engine().registerAlert(this);
  }

  /**
   * Unregister the alert from the Coral8 engine
   */
  public void unregister()
  {
    AlertsAdapter.getInstance().getCoral8Engine().unregisterAlert(this);
  }
}
