/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.delivery;

import java.util.Iterator;
import java.util.List;

import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.message.builder.TextMessageBuilder;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;

/**
 * @author Ralf Rantzau (rrantza@us.ibm.com)
 */
public class EmailDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String    NAME = "Email delivery method";
  private static final String    DESCRIPTION = "Writes notification into an email.";

  public static final IParameter TO_PARAMETER;
  public static final IParameter SUBJECT_PARAMETER;
  public static final IParameter FROM_PARAMETER;
  public static final IParameter FROM_PERSONAL_PARAMETER;
  public static final IParameter CC_PARAMETER;
  public static final IParameter BCC_PARAMETER;
  public static final IParameter PRIORITY_PARAMETER;
  public static final IParameter RECORD_FOLDER_PARAMETER;
  public static final IParameter MESSAGE_PARAMETER;
  
  private static final IParameter[] parameters;
  
  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    IAttributeType stringType = typeFactory.getStringType(50, 0);
    IAttributeType longStringType = typeFactory.getStringType(255, 0);
    
    TO_PARAMETER = new Parameter("To", stringType, true, null);
    SUBJECT_PARAMETER = new Parameter("Subject", stringType, true, null);
    FROM_PARAMETER = new Parameter("From", stringType, true, null);
    FROM_PERSONAL_PARAMETER = new Parameter("From personal", stringType, false, null);
    CC_PARAMETER = new Parameter("CC", stringType, false, null);
    BCC_PARAMETER = new Parameter("BCC", stringType, false, null);
    PRIORITY_PARAMETER = new Parameter("Priority", stringType, false, null);
    RECORD_FOLDER_PARAMETER = new Parameter("Record folder", stringType, false, null);
    MESSAGE_PARAMETER = new Parameter("Message", longStringType, true, null);
    
    parameters = new IParameter[] {
        TO_PARAMETER,
        SUBJECT_PARAMETER,
        FROM_PARAMETER,
        FROM_PERSONAL_PARAMETER,
        CC_PARAMETER,
        BCC_PARAMETER,
        PRIORITY_PARAMETER,
        RECORD_FOLDER_PARAMETER,
        MESSAGE_PARAMETER
      };
  }
  
  private TextMessageBuilder  messageBuilder;

  public EmailDeliveryMethod()
  {
    super(NAME, DESCRIPTION);

    messageBuilder = new TextMessageBuilder();      
  }

  public IParameter[] getParameters()
  {
    return parameters;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#processNotifications(java.util.List, com.ibm.rfidic.alerts.IDeliveryPoint)
   */
  public void processNotifications(List notifications, IDeliveryPoint deliveryPoint)
  {
    if( notifications == null || notifications.size() == 0 ) return;

    // we group the notifications that are sent for one subscription into one email.
    StringBuffer emailMessage = new StringBuffer();
    ISubscription subscription = null;
    Email email = null;
    
    for( Iterator ix = notifications.iterator(); ix.hasNext(); )
    {
      INotification notification = (INotification) ix.next();

      ISubscription sub = notification.getSubscription();
      if( sub != subscription )
      {
        if( email != null )
        {
          email.setText( emailMessage.toString() );
          email.send();
        }
        email = null;
        subscription = sub;
        emailMessage = new StringBuffer();
      }
      
      try
      {
        final IParameterValue[] parameterValues = notification.getDeliveryValues(deliveryPoint);
        final String message = (String) Utility.findValue( parameterValues, MESSAGE_PARAMETER ).getSQLValue();
        String text = messageBuilder.buildMessage(notification, message);
        
        emailMessage.append(text);
        
        if( email == null )
        {
          
          final String to = (String) Utility.findValue( parameterValues, TO_PARAMETER ).getSQLValue();
          final String subject = (String) Utility.findValue( parameterValues, SUBJECT_PARAMETER ).getSQLValue();
          final String from = (String) Utility.findValue( parameterValues, FROM_PARAMETER ).getSQLValue();
          
          IAttributeValue value = Utility.findValue( parameterValues, FROM_PERSONAL_PARAMETER );
          final String fromPersonal = value == null ? null : (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, CC_PARAMETER );
          final String cc = value == null ? null: (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, BCC_PARAMETER );
          final String bcc = value == null ? null: (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, PRIORITY_PARAMETER );
          final String priority = value == null ? null: (String) value.getSQLValue();
          value = Utility.findValue( parameterValues, RECORD_FOLDER_PARAMETER );
          final String recordFolder = value == null ? null: (String) value.getSQLValue();
        
          email = new Email(to, subject, from, fromPersonal, cc, bcc, priority, recordFolder, null);
        } 
      }
      catch(RFIDICException e)
      {
        e.printStackTrace();
      }
      
      if( email != null )
      {
        email.setText( emailMessage.toString() );
        email.send();
      }
    }
  }

  public String toString()
  {
    String text = "";
    text += "[";
    text += "id: " + getId() + ",\n";
    text += "className: " + this.getClass().getName() + ",\n";
    text += "name: " + getName() + ",\n";
    text += "description: " + getDescription() + ",\n";
    text += "]";
    return text;
  }

  public String[] getOutputChannels()
  {
    return null;
  }
}
