/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.rfidic.alerts.delivery;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.rfidic.alerts.AlertsAdapter;
import com.ibm.rfidic.alerts.Entity;
import com.ibm.rfidic.alerts.IDeliveryMethod;
import com.ibm.rfidic.alerts.IDeliveryPoint;
import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.util.Utility;
import com.ibm.rfidic.common.exceptions.RFIDICException;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;


public class ScheduleDeliveryMethod extends Entity implements IDeliveryMethod
{
  private static final String       OUTPUT_CHANNEL = "scheduledDelivery";
  private static final String       NAME = "Schedule delivery method";
  private static final String       DESCRIPTION = "Schedules the notifications and sends them to the subscriber in bulk at the specified time.";

  public static final IParameter    HOUR_PARAMETER;
  public static final IParameter    MINUTE_PARAMETER;
  private static final IParameter[] parameters;
  
  private static final String[]     outputChannels = new String[] { OUTPUT_CHANNEL };
  
  private static Map notificationMap = new HashMap();
  private static ScheduleThread scheduleThread = null;
  
  static
  {
    final TypeFactory typeFactory = TypeFactory.getInstance();
    final IAttributeType intType = typeFactory.getIntegerType();

    HOUR_PARAMETER = new Parameter("Scheduled hour", intType, true, null);
    MINUTE_PARAMETER = new Parameter("Scheduled minute", intType, true, null);

    parameters = new IParameter[] { HOUR_PARAMETER, MINUTE_PARAMETER };
  }
  
  static class ScheduleThread extends Thread
  {
    public void run()
    {
      // loop forever, later inside the loop we might decide to bail out
      while( true )
      {
        try
        {
          Calendar now = Calendar.getInstance();
          int currHour = now.get(Calendar.HOUR_OF_DAY);
          int currMinute = now.get(Calendar.MINUTE);
          int currTime = currHour * 60 + currMinute;
          
          synchronized ( notificationMap )
          {
            if( notificationMap.size() > 0 )
            {
              for( Iterator ix = notificationMap.keySet().iterator(); ix.hasNext(); )
              {
                IDeliveryPoint deliveryPoint = (IDeliveryPoint) ix.next();
                List notifications = (List) notificationMap.get(deliveryPoint);
                if( notifications.size() == 0 )
                {
                  ix.remove();
                }
                else
                {
                  // found some notifications pending, let's see if their scheduled time is now
                  // all notificaitons in the list belong to the same delivery point instance
                  // so that means that all of them have the same scheduled time
                  INotification notification = (INotification) notifications.get(0);
                  IParameterValue[] values = notification.getDeliveryValues(deliveryPoint);

                  IAttributeValue hourValue = Utility.findValue(values, HOUR_PARAMETER);
                  IAttributeValue minuteValue = Utility.findValue(values, MINUTE_PARAMETER);

                  int scheduledHour = ( (Integer) hourValue.getSQLValue() ).intValue();
                  int scheduledMinute = ( (Integer) minuteValue.getSQLValue() ).intValue();

                  Calendar scheduleTime = Calendar.getInstance();
                  scheduleTime.set(Calendar.HOUR_OF_DAY, scheduledHour);
                  scheduleTime.set(Calendar.MINUTE, scheduledMinute);
                  
                  // send the notifications further if we have reached or passed the scheduled time
                  // and only if the notifications have been received before the scheduled time 
                  if( currTime >= scheduledHour * 60 + scheduledMinute  &&
                      notification.getTimestamp().getTime() <= scheduleTime.getTimeInMillis() )
                  {
                    // first, remove these notifications from the map
                    ix.remove();

                    // find the delivery point that is mapped to the output channel
                    // and send the notifications to it
                    IDeliveryPoint nextDP = deliveryPoint.getDeliveryPoint(OUTPUT_CHANNEL);

                    if( nextDP != null )
                    {
                      nextDP.processNotifications(notifications);
                    }
                    else
                    {
                      throw new RFIDICException(this.getName() + ": Can't find the link for the outbound channel " + OUTPUT_CHANNEL);
                    }
                  }
                }
              }
            }
          }

          // sleep until next minute, but break it in 1 second chunks to be able to terminate if requested
          boolean terminate = false;
          int sec = 60 - Calendar.getInstance().get(Calendar.SECOND);
          for( int s = 0; s < sec; s++ )
          {
            // stop if there are no more pending notifications or if the alerts engine has stopped
            if( notificationMap.size() == 0 || AlertsAdapter.getInstance().isStarted() == false )
            {
              terminate = true;
              break;
            }
          
            Thread.sleep( 1000 );  // sleep one second
          }
          
          if( terminate ) break;
        }
        catch ( InterruptedException e )
        {
        }
        catch ( RFIDICException e )
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }
  }
  
  public ScheduleDeliveryMethod()
  {
    super(NAME, DESCRIPTION);
  }
  
  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#getOutputChannels()
   */
  public String[] getOutputChannels()
  {
    return outputChannels;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#getParameters()
   */
  public IParameter[] getParameters()
  {
    return parameters;
  }

  /* (non-Javadoc)
   * @see com.ibm.rfidic.alerts.IDeliveryMethod#processNotifications(java.util.List, com.ibm.rfidic.alerts.IDeliveryPoint)
   */
  public void processNotifications(List notifications, IDeliveryPoint deliveryPoint)
  {
    if( notifications == null ) return;
    
    synchronized ( notificationMap )
    {
      List notifs = (List) notificationMap.get(deliveryPoint);
      if( notifs == null )
      {
        notifs = new ArrayList();
        notificationMap.put(deliveryPoint, notifs);
      }
      notifs.addAll(notifications);
      
      // start the scheduling thread was not started yet or it has terminated
      if( scheduleThread == null || ! scheduleThread.isAlive() )
      {
        scheduleThread = new ScheduleThread();
        scheduleThread.start();
      }
    }
  }
}
