/**
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2006, 2008 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * (C) Copyright IBM Corp. and other(s) 2006, 2008  
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
*/ 
package com.ibm.rfidic.alerts.nativealerts;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.attribute.IAttribute;
import com.ibm.rfidic.common.epcis.EventConstants;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.mdm.IElement;
import com.ibm.rfidic.mdm.IVocabulary;
import com.ibm.rfidic.mdm.MasterDataManager;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;

/** 
 * This class is a sample Native Alert provided to show how a Native 
 * Alert should be built. It sends a notification when a product of a 
 * specific type is detected at a specific location. The product type 
 * and location are provided as input parameters and are also written
 * as Alert output parameters to the Notification object. 
 */
public class ProductAtLocationAlert extends SuperNativeAlert
{
	
  /* Declare arrays for the input and output parameters. */
  private IParameter[] inputParameters;
  private IParameter[] outputParameters;

  public ProductAtLocationAlert()
  {
    /* Call the super class constructor passing in the name and description of the 
     * alert. This text will appear in the Alerts Subscription wizard pages. */
    super("Product at location alert", 
          "Detects the presence of a product at a particular location."); 

    /* Create a type for the parameters. They will all be strings. The first 
     * parameter (255) is maximum size and the second is reserved size. */ 
    IAttributeType stringType = TypeFactory.getInstance().getStringType(255, 0);
    
    /* Declare an input parameter array for our alert definition. These parameters will
     * be provided by the subscription and entered by the user into the third page 
     * of the Alerts Subscription wizard. */
    this.inputParameters  = new IParameter[2];    
    
    this.inputParameters[0] = new Parameter("product",       // Name 
                                            stringType,      // Type - string             
                                            true,            // Required - yes
                                            null);           // Default value - none

    this.inputParameters[1] = new Parameter("location",      // Name 
                                            stringType,      // Type - string
                                            true,            // Required - yes
                                            null);           // Default value - none

    
    /* Declare an output parameter array for the Alert. These parameters will
     * be placed in the returned Notification and optionally used by the 
     * Delivery Method. */
    this.outputParameters = new IParameter[2];
    
    this.outputParameters[0] = new Parameter("product",       // Name 
                                             stringType,      // Type - string
                                             true,            // Required - yes
                                             null);           // Default value - none

    this.outputParameters[1] = new Parameter("location",      // Name 
                                             stringType,      // Type - string
                                             true,            // Required - yes
                                             null);           // Default value - none

  }

  /**
   * This method contains the condition to be evaluated for this Alert. It
   * returns a list of INotification objects or null if the alert condition 
   * is false.
   */
  protected List evaluateCondition(IEvent event, ISubscription subscription)
  {
    List returnNotifications = new ArrayList();
    
    /* Get input parameter values */	  
    IAttributeValue productAttrVal  = subscription.getAlertParameterValue("product").getValue();
    IAttributeValue locationAttrVal = subscription.getAlertParameterValue("location").getValue();
	  
    String paramProduct  = (String)productAttrVal.getSQLValue();
    String paramLocation = (String)locationAttrVal.getSQLValue();
	  
    /* Retrieve the standard EPCIS bizLocation attribute from the Event. */ 
    IAttributeValue bizLocValue   = getAttributeValue(event, EventConstants.bizLocation);  
    String          eventLocation = (String)bizLocValue.getSQLValue();

    /* Retrieve the extended (non standard EPCIS) attribute called ProductId
     * from the Event. The attribute name passed in needs to match the name of 
     * the attribute as specified in the event metadata. */ 
    IAttributeValue prodIdValue   = getAttributeValue(event, "ProductId");  
    Integer         eventProdId   = (Integer)prodIdValue.getSQLValue();
      
    /* Look up in masterdata the product name using the product id. */
    String eventProdName = getProductName(eventProdId.intValue()); 
      
    /* If the product and location in the event is the same as the product
     * and location input parameters the condition is true so return a 
     * notification. */ 
    if (eventProdName.equals(paramProduct) &&
        eventLocation.equals(paramLocation)) 
    {
      /* Create a parameter values array for use in the output notification. */
      final IParameterValue[] paramValues = new ParameterValue[2];
      
      /* Populate the array with parameters and their values */
      paramValues[0] = new ParameterValue(outputParameters[0], productAttrVal);
      paramValues[1] = new ParameterValue(outputParameters[1], locationAttrVal);
      
      /* Create and return the notification for this subscription. */  
      INotification notif = new Notification(subscription, paramValues);
      returnNotifications.add(notif);
        
      return returnNotifications;
    }
      
    /* Just return null if the condition is not true. */
    else
      return null;
  }
  
  /** Returns a product name from the Product master data entity based on 
   * an external product ID passed in. Returns "" if an error occurs or
   * the external ID cannot be found in the masterdata.
   */
  private String getProductName(int externalProductId)
  {
    /* Get an instance of the master data manager */
    MasterDataManager masterDM = MasterDataManager.getInstance();

    /* Initialize our return string. */
    String strVal = ""; 
    
    /* Lookup product id in master data and get values
     * Get the product vocabulary to retrieve the product element
     * If product vocabulary is null, there are no products */
    IVocabulary vocabulary = masterDM.getVocabulary("urn:epcglobal:epcis:vtype:Product"); 
    if (vocabulary != null)
    {
      IElement productElem = vocabulary.getElement(externalProductId);

      if (productElem != null)
      {
        /* Get string attribute value */
        IAttribute attr = productElem.getAttribute("urn.rfidic.product.core.name");
        if (attr != null)
          strVal = attr.getAttributeValue().toString();
      }
    }
    
    return strVal;
  }

  /** This method just returns the input parameters. 
   * There is no need to alter this method for Native Alerts. */
  public IParameter[] getInputParameters()
  {
    return inputParameters;
  }

  /** This method just returns the output parameters. 
   * There is no need to alter this method for Native Alerts. */
  public IParameter[] getOutputParameters()
  {
    return outputParameters;
  }
}
